import {
  IconButton,
  Paper,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  Tooltip,
} from "@mui/material";
import { SolarEnergyRouteContainer } from "../widgets/SolarEnergyRouteContainer";
import FileUploadIcon from "@mui/icons-material/FileUpload";
import { UploadUpdateDialog } from "../dialogs/UploadUpdateDialog";
import React from "react";
import { OTAAPI, OTAUpdate } from "../api/OTAApi";
import { AsyncWidget } from "../widgets/AsyncWidget";
import { filesize } from "filesize";

export function OTARoute(): React.ReactElement {
  const [list, setList] = React.useState<string[] | undefined>();
  const load = async () => {
    setList(await OTAAPI.SupportedPlatforms());
  };

  return (
    <AsyncWidget
      loadKey={1}
      ready={!!list}
      load={load}
      errMsg="Failed to load OTA screen!"
      build={() => <_OTARoute platforms={list!} />}
    />
  );
}

function _OTARoute(p: { platforms: Array<string> }): React.ReactElement {
  const key = React.useRef(1);
  const [showUploadDialog, setShowUploadDialog] = React.useState(false);

  const [list, setList] = React.useState<undefined | OTAUpdate[]>();

  const load = async () => {
    const list = await OTAAPI.ListOTAUpdates();
    list.sort((a, b) =>
      `${a.platform}#${a.version}`.localeCompare(`${b.platform}#${b.version}`)
    );
    list.reverse();
    setList(list);
  };

  const reload = async () => {
    key.current += 1;
    setList(undefined);
  };

  return (
    <SolarEnergyRouteContainer
      label="OTA"
      actions={
        <>
          <Tooltip title="Upload a new update">
            <IconButton onClick={() => setShowUploadDialog(true)}>
              <FileUploadIcon />
            </IconButton>
          </Tooltip>
        </>
      }
    >
      {showUploadDialog && (
        <UploadUpdateDialog
          platforms={p.platforms}
          onClose={() => setShowUploadDialog(false)}
          onCreated={() => {
            setShowUploadDialog(false);
            reload();
          }}
        />
      )}
      <AsyncWidget
        loadKey={key.current}
        ready={!!list}
        errMsg="Failed to load the list of OTA updates!"
        load={load}
        build={() => <_OTAList list={list!} />}
      />
    </SolarEnergyRouteContainer>
  );
}

function _OTAList(p: { list: OTAUpdate[] }): React.ReactElement {
  return (
    <TableContainer component={Paper}>
      <Table sx={{ minWidth: 650 }} aria-label="simple table">
        <TableHead>
          <TableRow>
            <TableCell align="center">Platform</TableCell>
            <TableCell align="center">Version</TableCell>
            <TableCell align="center">File size</TableCell>
          </TableRow>
        </TableHead>
        <TableBody>
          {p.list.map((row, num) => (
            <TableRow hover key={num}>
              <TableCell align="center">{row.platform}</TableCell>
              <TableCell align="center">{row.version}</TableCell>
              <TableCell align="center">{filesize(row.file_size)}</TableCell>
            </TableRow>
          ))}
        </TableBody>
      </Table>
    </TableContainer>
  );
}
